@@ -259,15 +259,31 @@ fn serve_request_stream<Ctx: Clone>(
 ) -> impl Stream<Item = Result<Message, warp::Error>> {
     let resp_stream = srv
         .serve_ws(ctx, payload, service_id)
-        .map(move |payload_result| match payload_result {
-            Ok(payload) => Outgoing::Next {
-                request_id: req_id,
-                payload,
-            },
-            Err(kind) => Outgoing::Error {
-                request_id: req_id,
-                kind,
-            },
+        .take_until_condition(|resp| future::ready(resp.is_err()))
+        .ready_chunks(128)
+        .flat_map(move |payload_results| {
+            let mut err = None;
+            let mut payload = Vec::with_capacity(payload_results.len());
+            for payload_result in payload_results {
+                match payload_result {
+                    Ok(value) => payload.push(value),
+                    Err(kind) => err = Some(kind), // always comes last
+                }
+            }
+            let mut res = Vec::with_capacity(1);
+            if !payload.is_empty() {
+                res.push(Outgoing::Next {
+                    request_id: req_id,
+                    payload,
+                });
+            }
+            if let Some(kind) = err {
+                res.push(Outgoing::Error {
+                    request_id: req_id,
+                    kind,
+                });
+            }
+            stream::iter(res)
         });
 
     AssertUnwindSafe(resp_stream)
@@ -403,12 +419,39 @@ mod tests {
         }
     }
 
+    // Copy of Outgoing that uses Value over Box<RawValue>
+    // Needed due to https://github.com/serde-rs/json/issues/779
+    #[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
+    #[serde(tag = "type")]
+    #[serde(rename_all = "camelCase")]
+    pub enum OutgoingAst {
+        #[serde(rename_all = "camelCase")]
+        Next {
+            request_id: ReqId,
+            payload: Vec<Value>,
+        },
+        #[serde(rename_all = "camelCase")]
+        Complete { request_id: ReqId },
+        #[serde(rename_all = "camelCase")]
+        Error { request_id: ReqId, kind: ErrorKind },
+    }
+
+    impl OutgoingAst {
+        pub fn request_id(&self) -> ReqId {
+            match self {
+                OutgoingAst::Next { request_id, .. } => *request_id,
+                OutgoingAst::Complete { request_id, .. } => *request_id,
+                OutgoingAst::Error { request_id, .. } => *request_id,
+            }
+        }
+    }
+
     fn test_client<Req: Serialize, Resp: DeserializeOwned>(
         addr: SocketAddr,
         endpoint: &str,
         id: u64,
         req: Req,
-    ) -> (Vec<Resp>, Outgoing) {
+    ) -> (Vec<Resp>, OutgoingAst) {
         let addr = format!("ws://{}/test_ws", addr);
         let client = ClientBuilder::new(&*addr)
             .expect("Could not setup client")
@@ -430,14 +473,14 @@ mod tests {
             .send_message(&OwnedMessage::Text(req_env_json))
             .expect("Could not send request");
 
-        let mut completion: Option<Outgoing> = None;
+        let mut completion: Option<OutgoingAst> = None;
 
         let msgs = receiver
             .incoming_messages()
             .filter_map(move |msg| {
                 let msg_ok = msg.expect("Expected message but got websocket error");
                 if let OwnedMessage::Text(raw_resp) = msg_ok {
-                    let resp_env: Outgoing = serde_json::from_str(&*raw_resp)
+                    let resp_env: OutgoingAst = serde_json::from_str(&*raw_resp)
                         .expect("Could not deserialize response envelope");
                     if resp_env.request_id().0 == id {
                         Some(resp_env)
@@ -449,21 +492,24 @@ mod tests {
                 }
             })
             .take_while(|env| {
-                if let Outgoing::Next { .. } = env {
+                if let OutgoingAst::Next { .. } = env {
                     true
                 } else {
                     completion = Some(env.clone());
                     false
                 }
             })
-            .filter_map(|env| {
-                if let Outgoing::Next { payload, .. } = env {
-                    Some(
-                        serde_json::from_value::<Resp>(payload)
-                            .expect("Could not deserialize response"),
-                    )
+            .flat_map(|env| {
+                if let OutgoingAst::Next { payload, .. } = env {
+                    payload
+                        .into_iter()
+                        .map(|p| {
+                            serde_json::from_value::<Resp>(p)
+                                .expect("Could not deserialize response")
+                        })
+                        .collect()
                 } else {
-                    None
+                    vec![]
                 }
             })
             .collect();
@@ -561,7 +607,7 @@ mod tests {
 
         assert_eq!(
             completion,
-            Outgoing::Error {
+            OutgoingAst::Error {
                 request_id: ReqId(49),
                 kind: ErrorKind::UnknownEndpoint {
                     endpoint: "no_such_service".to_string(),
@@ -586,7 +632,7 @@ mod tests {
 
         assert_eq!(msgs, vec![]);
 
-        if let Outgoing::Error {
+        if let OutgoingAst::Error {
             request_id: ReqId(49),
             kind: ErrorKind::BadRequest { message },
         } = completion
@@ -612,7 +658,7 @@ mod tests {
 
         assert_eq!(
             completion,
-            Outgoing::Error {
+            OutgoingAst::Error {
                 request_id: ReqId(49),
                 kind: ErrorKind::ServiceError {
                     value: Value::String("Test reason".to_string())
@@ -631,7 +677,7 @@ mod tests {
 
         assert_eq!(
             completion,
-            Outgoing::Error {
+            OutgoingAst::Error {
                 request_id: ReqId(49),
                 kind: ErrorKind::InternalError,
             }
